# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .row_metadata import RowMetadata
from .message_metadata import MessageMetadata

__all__ = ["AttachmentMetadata", "Source", "Page"]

Source: TypeAlias = Union[MessageMetadata, RowMetadata]


class Page(BaseModel):
    page_number: float = FieldInfo(alias="pageNumber")
    """The page number of the attachment."""

    url: str
    """The url of the blob of the page."""


class AttachmentMetadata(BaseModel):
    media_type: str
    """The media type of the attachment."""

    name: str
    """The name of the attachment."""

    object: Literal["attachment_metadata"]
    """The object type, which is always `attachment_metadata`."""

    source: Source
    """The source of the attachment."""

    url: str
    """The url of the blob of the attachment."""

    page: Optional[Page] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .search_result_resource import SearchResultResource

__all__ = ["SearchResultItem"]


class SearchResultItem(BaseModel):
    content: List[str]
    """Text snippets relevant to the search query."""

    object: Literal["search_result_item"]
    """The object type, which is always `search_result_item`."""

    resource: SearchResultResource
    """The resource that matched the search query."""

    score: float
    """The score of the item, between 0 and 1."""

    updated_at: datetime
    """The date and time the item was last updated."""

    summary: Optional[str] = None
    """The summary of the item."""

    title: Optional[str] = None
    """The title of the item."""



# File created for custom code changes based on our OpenAPI spec.

from typing import Union
from typing_extensions import Literal, Annotated, TypeAlias

from ..._models import BaseModel
from ..._utils._transform import PropertyInfo

__all__ = [
    "Event",
    "EventConverseStatusEvent",
    "EventConverseStatusEventData",
    "EventConverseContentMessageDeltaEvent",
    "EventConverseContentMessageDeltaEventData",
    "EventConverseContentMessageDeltaEventDataDelta",
    "AgentStreamEvent",
]


class EventConverseStatusEventData(BaseModel):
    agent_id: str
    """The ID of the agent used for the converse."""

    conversation_id: str
    """The ID of the agent conversation."""

    status: str
    """Name of the status"""


class EventConverseStatusEvent(BaseModel):
    data: EventConverseStatusEventData

    event: Literal["start", "end"]


class EventConverseContentMessageDeltaEventDataDelta(BaseModel):
    text: str

    event: Literal["text"]


class EventConverseContentMessageDeltaEventData(BaseModel):
    delta: EventConverseContentMessageDeltaEventDataDelta
    """Delta of the response message produced by the agent."""


class EventConverseContentMessageDeltaEvent(BaseModel):
    data: EventConverseContentMessageDeltaEventData

    event: Literal["delta"]
    """Type of the event which is always delta"""


Event: TypeAlias = Union[EventConverseStatusEvent, EventConverseContentMessageDeltaEvent]

AgentStreamEvent: TypeAlias = Annotated[
    Union[EventConverseStatusEvent, EventConverseContentMessageDeltaEvent], PropertyInfo(discriminator="event")
]



# File created for custom code changes based on our OpenAPI spec.

from .converse_response_events import (
    Event,
    AgentStreamEvent,
    EventConverseStatusEvent,
    EventConverseStatusEventData,
    EventConverseContentMessageDeltaEvent,
    EventConverseContentMessageDeltaEventData,
    EventConverseContentMessageDeltaEventDataDelta,
)

__all__ = [
    "AgentStreamEvent",
    "Event",
    "EventConverseContentMessageDeltaEvent",
    "EventConverseContentMessageDeltaEventData",
    "EventConverseContentMessageDeltaEventDataDelta",
    "EventConverseStatusEvent",
    "EventConverseStatusEventData",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .knowledge import (
    KnowledgeResource,
    AsyncKnowledgeResource,
    KnowledgeResourceWithRawResponse,
    AsyncKnowledgeResourceWithRawResponse,
    KnowledgeResourceWithStreamingResponse,
    AsyncKnowledgeResourceWithStreamingResponse,
)

__all__ = [
    "KnowledgeResource",
    "AsyncKnowledgeResource",
    "KnowledgeResourceWithRawResponse",
    "AsyncKnowledgeResourceWithRawResponse",
    "KnowledgeResourceWithStreamingResponse",
    "AsyncKnowledgeResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel
from .knowledge_metadata import KnowledgeMetadata

__all__ = ["TableMetadata"]


class TableMetadata(BaseModel):
    id: str
    """The unique identifier for the table."""

    knowledge: KnowledgeMetadata
    """The knowledge object that the table belongs to."""

    name: str
    """The name of the table."""

    object: Literal["table_metadata"]
    """The object type, which is always `table_metadata`."""

    url: str
    """The url of the table."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["KnowledgeCreateParams"]


class KnowledgeCreateParams(TypedDict, total=False):
    files: Required[List[FileTypes]]
    """The files to be uploaded and learned.

    Supported media types are `pdf`, `json`, `csv`, `text`, `png`, `jpeg`, `excel`,
    `google sheets`.
    """

    name: str
    """The name of the knowledge."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import ConverseResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestClient:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_converse(self, client: Datagrid) -> None:
        client_ = client.converse(
            prompt="string",
        )
        assert_matches_type(ConverseResponse, client_, path=["response"])

    @parametrize
    def test_method_converse_with_all_params(self, client: Datagrid) -> None:
        client_ = client.converse(
            prompt="string",
            agent_id="agent_id",
            config={
                "agent_model": "magpie-1",
                "agent_tools": ["data_analysis"],
                "knowledge_ids": ["string"],
                "llm_model": "gemini-1.5-flash-001",
                "system_prompt": "system_prompt",
            },
            conversation_id="conversation_id",
            generate_citations=True,
            stream=False,
        )
        assert_matches_type(ConverseResponse, client_, path=["response"])

    @parametrize
    def test_raw_response_converse(self, client: Datagrid) -> None:
        response = client.with_raw_response.converse(
            prompt="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client_ = response.parse()
        assert_matches_type(ConverseResponse, client_, path=["response"])

    @parametrize
    def test_streaming_response_converse(self, client: Datagrid) -> None:
        with client.with_streaming_response.converse(
            prompt="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client_ = response.parse()
            assert_matches_type(ConverseResponse, client_, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncClient:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_converse(self, async_client: AsyncDatagrid) -> None:
        client = await async_client.converse(
            prompt="string",
        )
        assert_matches_type(ConverseResponse, client, path=["response"])

    @parametrize
    async def test_method_converse_with_all_params(self, async_client: AsyncDatagrid) -> None:
        client = await async_client.converse(
            prompt="string",
            agent_id="agent_id",
            config={
                "agent_model": "magpie-1",
                "agent_tools": ["data_analysis"],
                "knowledge_ids": ["string"],
                "llm_model": "gemini-1.5-flash-001",
                "system_prompt": "system_prompt",
            },
            conversation_id="conversation_id",
            generate_citations=True,
            stream=False,
        )
        assert_matches_type(ConverseResponse, client, path=["response"])

    @parametrize
    async def test_raw_response_converse(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.with_raw_response.converse(
            prompt="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client = await response.parse()
        assert_matches_type(ConverseResponse, client, path=["response"])

    @parametrize
    async def test_streaming_response_converse(self, async_client: AsyncDatagrid) -> None:
        async with async_client.with_streaming_response.converse(
            prompt="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client = await response.parse()
            assert_matches_type(ConverseResponse, client, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import (
    Knowledge,
    KnowledgeUpdateResponse,
)
from datagrid_ai.pagination import SyncCursorIDPage, AsyncCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledge:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])
    skip = pytest.mark.skip("Problematic tests - issue with Prism and array of files (multipart/form-data)")

    @parametrize
    @skip
    def test_method_create(self, client: Datagrid) -> None:
        knowledge = client.knowledge.create(
            files=[b"raw file contents"],
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    def test_method_create_with_all_params(self, client: Datagrid) -> None:
        knowledge = client.knowledge.create(
            files=[b"raw file contents"],
            name="name",
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    def test_raw_response_create(self, client: Datagrid) -> None:
        response = client.knowledge.with_raw_response.create(
            files=[b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = response.parse()
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    def test_streaming_response_create(self, client: Datagrid) -> None:
        with client.knowledge.with_streaming_response.create(
            files=[b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = response.parse()
            assert_matches_type(Knowledge, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Datagrid) -> None:
        knowledge = client.knowledge.retrieve(
            "knowledge_id",
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Datagrid) -> None:
        response = client.knowledge.with_raw_response.retrieve(
            "knowledge_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = response.parse()
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Datagrid) -> None:
        with client.knowledge.with_streaming_response.retrieve(
            "knowledge_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = response.parse()
            assert_matches_type(Knowledge, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            client.knowledge.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Datagrid) -> None:
        knowledge = client.knowledge.update(
            knowledge_id="knowledge_id",
            name="name",
        )
        assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Datagrid) -> None:
        response = client.knowledge.with_raw_response.update(
            knowledge_id="knowledge_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = response.parse()
        assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Datagrid) -> None:
        with client.knowledge.with_streaming_response.update(
            knowledge_id="knowledge_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = response.parse()
            assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            client.knowledge.with_raw_response.update(
                knowledge_id="",
                name="name",
            )

    @parametrize
    def test_method_list(self, client: Datagrid) -> None:
        knowledge = client.knowledge.list()
        assert_matches_type(SyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Datagrid) -> None:
        knowledge = client.knowledge.list(
            after="after",
            before="before",
            limit=1,
        )
        assert_matches_type(SyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Datagrid) -> None:
        response = client.knowledge.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = response.parse()
        assert_matches_type(SyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Datagrid) -> None:
        with client.knowledge.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = response.parse()
            assert_matches_type(SyncCursorIDPage[Knowledge], knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Datagrid) -> None:
        knowledge = client.knowledge.delete(
            "knowledge_id",
        )
        assert knowledge is None

    @parametrize
    def test_raw_response_delete(self, client: Datagrid) -> None:
        response = client.knowledge.with_raw_response.delete(
            "knowledge_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = response.parse()
        assert knowledge is None

    @parametrize
    def test_streaming_response_delete(self, client: Datagrid) -> None:
        with client.knowledge.with_streaming_response.delete(
            "knowledge_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = response.parse()
            assert knowledge is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            client.knowledge.with_raw_response.delete(
                "",
            )


class TestAsyncKnowledge:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])
    skip = pytest.mark.skip("Problematic tests - issue with Prism and array of files (multipart/form-data)")

    @parametrize
    @skip
    async def test_method_create(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.create(
            files=[b"raw file contents"],
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    async def test_method_create_with_all_params(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.create(
            files=[b"raw file contents"],
            name="name",
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    async def test_raw_response_create(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.knowledge.with_raw_response.create(
            files=[b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = await response.parse()
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    @skip
    async def test_streaming_response_create(self, async_client: AsyncDatagrid) -> None:
        async with async_client.knowledge.with_streaming_response.create(
            files=[b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = await response.parse()
            assert_matches_type(Knowledge, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.retrieve(
            "knowledge_id",
        )
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.knowledge.with_raw_response.retrieve(
            "knowledge_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = await response.parse()
        assert_matches_type(Knowledge, knowledge, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        async with async_client.knowledge.with_streaming_response.retrieve(
            "knowledge_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = await response.parse()
            assert_matches_type(Knowledge, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            await async_client.knowledge.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.update(
            knowledge_id="knowledge_id",
            name="name",
        )
        assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.knowledge.with_raw_response.update(
            knowledge_id="knowledge_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = await response.parse()
        assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDatagrid) -> None:
        async with async_client.knowledge.with_streaming_response.update(
            knowledge_id="knowledge_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = await response.parse()
            assert_matches_type(KnowledgeUpdateResponse, knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            await async_client.knowledge.with_raw_response.update(
                knowledge_id="",
                name="name",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.list()
        assert_matches_type(AsyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.list(
            after="after",
            before="before",
            limit=1,
        )
        assert_matches_type(AsyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.knowledge.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = await response.parse()
        assert_matches_type(AsyncCursorIDPage[Knowledge], knowledge, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDatagrid) -> None:
        async with async_client.knowledge.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = await response.parse()
            assert_matches_type(AsyncCursorIDPage[Knowledge], knowledge, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncDatagrid) -> None:
        knowledge = await async_client.knowledge.delete(
            "knowledge_id",
        )
        assert knowledge is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.knowledge.with_raw_response.delete(
            "knowledge_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge = await response.parse()
        assert knowledge is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncDatagrid) -> None:
        async with async_client.knowledge.with_streaming_response.delete(
            "knowledge_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge = await response.parse()
            assert knowledge is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_id` but received ''"):
            await async_client.knowledge.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SearchResultResourceType"]

SearchResultResourceType: TypeAlias = Literal[
    "knowledge_metadata", "table_metadata", "row_metadata", "attachment_metadata", "message_metadata"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Knowledge", "RowCounts"]


class RowCounts(BaseModel):
    completed: float
    """The number of rows successfully learned."""

    failed: float
    """The number of rows that failed to be processed for learning."""

    total: float
    """The total number of rows in the knowledge."""


class Knowledge(BaseModel):
    id: str
    """The knowledge identifier, which can be referenced in the API endpoints."""

    created_at: datetime
    """The ISO string for when the knowledge was created."""

    name: str
    """The name of the knowledge"""

    object: Literal["knowledge"]
    """The object type, which is always `knowledge`."""

    row_counts: RowCounts
    """Row count statistics for the knowledge."""

    status: Literal["pending", "partial", "ready"]
    """
    The current knowledge status can be one of three values: `pending`, `partial`,
    or `ready`. `pending` indicates that the knowledge is awaiting learning and will
    not be used by the agent when responding. `partial` indicates that the knowledge
    is partially learned. The agent may use some aspects of it when responding.
    `ready` indicates that the knowledge is fully learned and will be completely
    utilized in responses.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping, Iterable, Optional, overload
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from .lib import sse_converse
from .types import client_converse_params
from ._types import (
    NOT_GIVEN,
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import (
    is_given,
    maybe_transform,
    get_async_library,
    async_maybe_transform,
)
from ._version import __version__
from ._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .resources import files, search, knowledge
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import DatagridError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .types.converse_response import ConverseResponse

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Datagrid",
    "AsyncDatagrid",
    "Client",
    "AsyncClient",
]


class Datagrid(SyncAPIClient):
    knowledge: knowledge.KnowledgeResource
    files: files.FilesResource
    search: search.SearchResource
    with_raw_response: DatagridWithRawResponse
    with_streaming_response: DatagridWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Datagrid client instance.

        This automatically infers the `api_key` argument from the `DATAGRID_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("DATAGRID_API_KEY")
        if api_key is None:
            raise DatagridError(
                "The api_key client option must be set either by passing api_key to the client or by setting the DATAGRID_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("DATAGRID_BASE_URL")
        if base_url is None:
            base_url = f"https://api.datagrid.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.knowledge = knowledge.KnowledgeResource(self)
        self.files = files.FilesResource(self)
        self.search = search.SearchResource(self)
        self.with_raw_response = DatagridWithRawResponse(self)
        self.with_streaming_response = DatagridWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse: ...

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Stream[sse_converse.AgentStreamEvent]: ...

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse | Stream[sse_converse.AgentStreamEvent]: ...

    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse | Stream[sse_converse.AgentStreamEvent]:
        """
        Converse with an AI Agent

        Args:
          prompt: A text prompt to send to the agent.

          agent_id: The ID of the agent that should be used for the converse. If both agent_id and
              conversation_id aren't provided - the new agent is created.

          config: The config that overrides the default config of the agent for that converse.

          conversation_id: The ID of the present conversation to use. If it's not provided - a new
              conversation will be created.

          generate_citations: Determines whether the response should include citations. When enabled, the
              agent will generate citations for factual statements.

          stream: Determines the response type of the converse. Response is the Server-Sent Events
              if stream is set to true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self.post(
            "/converse",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "agent_id": agent_id,
                    "config": config,
                    "conversation_id": conversation_id,
                    "generate_citations": generate_citations,
                    "stream": stream,
                },
                client_converse_params.ClientConverseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConverseResponse,
            stream=stream or False,
            stream_cls=Stream[sse_converse.AgentStreamEvent],
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncDatagrid(AsyncAPIClient):
    knowledge: knowledge.AsyncKnowledgeResource
    files: files.AsyncFilesResource
    search: search.AsyncSearchResource
    with_raw_response: AsyncDatagridWithRawResponse
    with_streaming_response: AsyncDatagridWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncDatagrid client instance.

        This automatically infers the `api_key` argument from the `DATAGRID_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("DATAGRID_API_KEY")
        if api_key is None:
            raise DatagridError(
                "The api_key client option must be set either by passing api_key to the client or by setting the DATAGRID_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("DATAGRID_BASE_URL")
        if base_url is None:
            base_url = f"https://api.datagrid.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.knowledge = knowledge.AsyncKnowledgeResource(self)
        self.files = files.AsyncFilesResource(self)
        self.search = search.AsyncSearchResource(self)
        self.with_raw_response = AsyncDatagridWithRawResponse(self)
        self.with_streaming_response = AsyncDatagridWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse: ...

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncStream[sse_converse.AgentStreamEvent]: ...

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse | AsyncStream[sse_converse.AgentStreamEvent]: ...

    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: str | NotGiven = NOT_GIVEN,
        config: client_converse_params.Config | NotGiven = NOT_GIVEN,
        conversation_id: str | NotGiven = NOT_GIVEN,
        generate_citations: bool | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConverseResponse | AsyncStream[sse_converse.AgentStreamEvent]:
        """
        Converse with an AI Agent

        Args:
          prompt: A text prompt to send to the agent.

          agent_id: The ID of the agent that should be used for the converse. If both agent_id and
              conversation_id aren't provided - the new agent is created.

          config: The config that overrides the default config of the agent for that converse.

          conversation_id: The ID of the present conversation to use. If it's not provided - a new
              conversation will be created.

          generate_citations: Determines whether the response should include citations. When enabled, the
              agent will generate citations for factual statements.

          stream: Determines the response type of the converse. Response is the Server-Sent Events
              if stream is set to true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self.post(
            "/converse",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "agent_id": agent_id,
                    "config": config,
                    "conversation_id": conversation_id,
                    "generate_citations": generate_citations,
                    "stream": stream,
                },
                client_converse_params.ClientConverseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConverseResponse,
            stream=stream or False,
            stream_cls=AsyncStream[sse_converse.AgentStreamEvent],
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class DatagridWithRawResponse:
    def __init__(self, client: Datagrid) -> None:
        self.knowledge = knowledge.KnowledgeResourceWithRawResponse(client.knowledge)
        self.files = files.FilesResourceWithRawResponse(client.files)
        self.search = search.SearchResourceWithRawResponse(client.search)

        self.converse = to_raw_response_wrapper(
            client.converse,
        )


class AsyncDatagridWithRawResponse:
    def __init__(self, client: AsyncDatagrid) -> None:
        self.knowledge = knowledge.AsyncKnowledgeResourceWithRawResponse(client.knowledge)
        self.files = files.AsyncFilesResourceWithRawResponse(client.files)
        self.search = search.AsyncSearchResourceWithRawResponse(client.search)

        self.converse = async_to_raw_response_wrapper(
            client.converse,
        )


class DatagridWithStreamedResponse:
    def __init__(self, client: Datagrid) -> None:
        self.knowledge = knowledge.KnowledgeResourceWithStreamingResponse(client.knowledge)
        self.files = files.FilesResourceWithStreamingResponse(client.files)
        self.search = search.SearchResourceWithStreamingResponse(client.search)

        self.converse = to_streamed_response_wrapper(
            client.converse,
        )


class AsyncDatagridWithStreamedResponse:
    def __init__(self, client: AsyncDatagrid) -> None:
        self.knowledge = knowledge.AsyncKnowledgeResourceWithStreamingResponse(client.knowledge)
        self.files = files.AsyncFilesResourceWithStreamingResponse(client.files)
        self.search = search.AsyncSearchResourceWithStreamingResponse(client.search)

        self.converse = async_to_streamed_response_wrapper(
            client.converse,
        )


Client = Datagrid

AsyncClient = AsyncDatagrid



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import SearchResultItem
from datagrid_ai.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_search(self, client: Datagrid) -> None:
        search = client.search.search(
            query="query",
        )
        assert_matches_type(SyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: Datagrid) -> None:
        search = client.search.search(
            query="query",
            limit=1,
            next="next",
        )
        assert_matches_type(SyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: Datagrid) -> None:
        response = client.search.with_raw_response.search(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: Datagrid) -> None:
        with client.search.with_streaming_response.search(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SyncCursorPage[SearchResultItem], search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_search(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search(
            query="query",
        )
        assert_matches_type(AsyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search(
            query="query",
            limit=1,
            next="next",
        )
        assert_matches_type(AsyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.search.with_raw_response.search(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(AsyncCursorPage[SearchResultItem], search, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncDatagrid) -> None:
        async with async_client.search.with_streaming_response.search(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(AsyncCursorPage[SearchResultItem], search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Any, List, Generic, TypeVar, Optional, cast
from typing_extensions import Protocol, override, runtime_checkable

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncCursorPage", "AsyncCursorPage", "SyncCursorIDPage", "AsyncCursorIDPage"]

_T = TypeVar("_T")


@runtime_checkable
class CursorIDPageItem(Protocol):
    id: str


class SyncCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    next: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next = self.next
        if not next:
            return None

        return PageInfo(params={"next": next})


class AsyncCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    next: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next = self.next
        if not next:
            return None

        return PageInfo(params={"next": next})


class SyncCursorIDPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        data = self.data
        if not data:
            return None

        if is_forwards:
            item = cast(Any, data[-1])
            if not isinstance(item, CursorIDPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.data[0])
            if not isinstance(item, CursorIDPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})


class AsyncCursorIDPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        data = self.data
        if not data:
            return None

        if is_forwards:
            item = cast(Any, data[-1])
            if not isinstance(item, CursorIDPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.data[0])
            if not isinstance(item, CursorIDPageItem) or item.id is None:  # pyright: ignore[reportUnnecessaryComparison]
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MessageMetadata", "Conversation", "ConversationNavigationItem"]


class ConversationNavigationItem(BaseModel):
    id: str
    """The id of the navigation item."""

    item_type: str
    """The type of the navigation item."""

    name: str
    """The name of the navigation item."""

    object: Literal["navigation_item_metadata"]
    """The object type, which is always `navigation_item_metadata`."""

    url: str
    """The url of the navigation item."""

    emoticon: Optional[str] = None
    """The emoticon of the navigation item."""

    source_media_type: Optional[str] = None
    """The media type of the navigation item if known."""


class Conversation(BaseModel):
    id: str
    """The id of the conversation."""

    name: str
    """The name of the conversation."""

    navigation_item: ConversationNavigationItem
    """The navigation item of the conversation."""

    object: Literal["conversation_metadata"]
    """The object type, which is always `conversation_metadata`."""

    url: str
    """The url of the conversation."""


class MessageMetadata(BaseModel):
    id: str
    """The id of the message."""

    author_id: str
    """The identifier of the message author (either a user ID or agent ID)."""

    author_type: Literal["user", "agent"]
    """Indicates whether the author is a user or an agent."""

    conversation: Conversation
    """The conversation that the message belongs to."""

    object: Literal["message_metadata"]
    """The object type, which is always `message_metadata`."""

    url: str
    """The url of the message."""

    author_name: Optional[str] = None
    """The pretty name of the author of the message."""
